`timescale 1ns / 1ps

module usb_tb();

reg clk, clk_48mhz;
reg reset;

wire pin_usbp, pin_usbn;
wire pin_pu;

wire [7:0] rx_data;
wire rx_strobe;
reg [7:0] tx_data;
reg tx_strobe;

assign pin_usbp = pin_pu ?  tx_strobe :  rx_strobe;
assign pin_usbn = pin_pu ? ~tx_strobe : ~rx_strobe;

usb_uart dut
(
	.clk(clk),
	.clk_48mhz(clk_48mhz),
	.reset(reset),
	
	.pin_usbp(pin_usbp),
	.pin_usbn(pin_usbn),
	.pin_pu(pin_pu),
	
	.rx_data(rx_data),
	.rx_strobe(rx_strobe),
	.tx_data(tx_data),
	.tx_strobe(tx_strobe)
);

initial begin
 	$dumpfile("usb_tb.vcd"); 
 	$dumpvars; 
end 

initial
begin
	clk = 0;
	clk_48mhz = 0;
	reset = 0;
	tx_data = 0;
	tx_strobe = 0;

	@(posedge clk_48mhz);
	reset = 1;

	@(posedge clk_48mhz);
	@(posedge clk_48mhz);

	reset = 0;
		
	@(posedge clk_48mhz);
	tx_strobe = 1;
	tx_data = 1;
	
	@(posedge clk_48mhz);
	tx_data = 2;
	
	@(posedge clk_48mhz);
	tx_data = 3;
	
	#50 $finish;
end

always #5  clk = !clk; 
always #10  clk_48mhz = !clk_48mhz; 

endmodule
